/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.project.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.project.pojo.vo.WorkFocusProgressInfoVO;
import org.springblade.modules.project.pojo.vo.WorkFocusProgressListVO;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.project.pojo.entity.WorkFocusProgressEntity;
import org.springblade.modules.project.pojo.vo.WorkFocusProgressVO;
import org.springblade.modules.project.pojo.excel.WorkFocusProgressExcel;
import org.springblade.modules.project.wrapper.WorkFocusProgressWrapper;
import org.springblade.modules.project.service.IWorkFocusProgressService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 工作要点完成情况表 控制器
 *
 * @author BladeX
 * @since 2024-04-09
 */
@RestController
@AllArgsConstructor
@RequestMapping("blade-workFocusProgress/workFocusProgress")
@Tag(name = "工作要点完成情况表", description = "工作要点完成情况表接口")
public class WorkFocusProgressController extends BladeController {

	private final IWorkFocusProgressService workFocusProgressService;
	private JdbcTemplate jdbcTemplate;
/*
	*//**
	 * 工作要点完成情况表 详情
	 *//*
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入workFocusProgress")
	public R<WorkFocusProgressVO> detail(WorkFocusProgressEntity workFocusProgress) {
		WorkFocusProgressEntity detail = workFocusProgressService.getOne(Condition.getQueryWrapper(workFocusProgress));
		return R.data(WorkFocusProgressWrapper.build().entityVO(detail));
	}
	*//**
	 * 工作要点完成情况表 分页
	 *//*
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "分页", description  = "传入workFocusProgress")
	public R<IPage<WorkFocusProgressVO>> list(@Parameter(hidden = true) @RequestParam Map<String, Object> workFocusProgress, Query query) {
		IPage<WorkFocusProgressEntity> pages = workFocusProgressService.page(Condition.getPage(query), Condition.getQueryWrapper(workFocusProgress, WorkFocusProgressEntity.class));
		return R.data(WorkFocusProgressWrapper.build().pageVO(pages));
	}

	*//**
	 * 工作要点完成情况表 自定义分页
	 *//*
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "自定义分页", description  = "传入workFocusProgress")
	public R<IPage<WorkFocusProgressVO>> page(WorkFocusProgressVO workFocusProgress, Query query) {
		IPage<WorkFocusProgressVO> pages = workFocusProgressService.selectWorkFocusProgressPage(Condition.getPage(query), workFocusProgress);
		return R.data(pages);
	}

	*//**
	 * 工作要点完成情况表 新增
	 *//*
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增", description  = "传入workFocusProgress")
	public R save(@Valid @RequestBody WorkFocusProgressEntity workFocusProgress) {
		return R.status(workFocusProgressService.save(workFocusProgress));
	}

	*//**
	 * 工作要点完成情况表 修改
	 *//*
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入workFocusProgress")
	public R update(@Valid @RequestBody WorkFocusProgressEntity workFocusProgress) {
		return R.status(workFocusProgressService.updateById(workFocusProgress));
	}

	*//**
	 * 工作要点完成情况表 新增或修改
	 *//*
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "新增或修改", description  = "传入workFocusProgress")
	public R submit(@Valid @RequestBody WorkFocusProgressEntity workFocusProgress) {
		return R.status(workFocusProgressService.saveOrUpdate(workFocusProgress));
	}

	*//**
	 * 工作要点完成情况表 删除
	 *//*
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "逻辑删除", description  = "传入ids")
	public R remove(@Parameter(name = "主键集合", required = true) @RequestParam String ids) {
		return R.status(workFocusProgressService.deleteLogic(Func.toLongList(ids)));
	}


	*//**
	 * 导出数据
	 *//*
	@PreAuth(RoleConstant.HAS_ROLE_ADMIN)
	@GetMapping("/export-workFocusProgress")
	@ApiOperationSupport(order = 9)
	@Operation(summary = "导出数据", description  = "传入workFocusProgress")
	public void exportWorkFocusProgress(@Parameter(hidden = true) @RequestParam Map<String, Object> workFocusProgress, BladeUser bladeUser, HttpServletResponse response) {
		QueryWrapper<WorkFocusProgressEntity> queryWrapper = Condition.getQueryWrapper(workFocusProgress, WorkFocusProgressEntity.class);
		//if (!AuthUtil.isAdministrator()) {
		//	queryWrapper.lambda().eq(WorkFocusProgress::getTenantId, bladeUser.getTenantId());
		//}
		queryWrapper.lambda().eq(WorkFocusProgressEntity::getIsDeleted, BladeConstant.DB_NOT_DELETED);
		List<WorkFocusProgressExcel> list = workFocusProgressService.exportWorkFocusProgress(queryWrapper);
		ExcelUtil.export(response, "工作要点完成情况表数据" + DateUtil.time(), "工作要点完成情况表数据表", list, WorkFocusProgressExcel.class);
	}*/

	@GetMapping("/queryWorkFocusProgressInfoById")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入workFocusProgress")
	public R<WorkFocusProgressListVO> queryWorkFocusProgressInfoById(WorkFocusProgressListVO workFocusProgress) {
		WorkFocusProgressListVO detail = workFocusProgressService.queryWorkFocusProgressInfoById(workFocusProgress);
		return R.data(detail);
	}

	@PostMapping("/submit")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "新增或修改", description  = "传入workFocusProgress")
	public R submit(@Valid @RequestBody WorkFocusProgressListVO workFocusProgress) {
		if(workFocusProgress!=null){
			List<WorkFocusProgressInfoVO> table = workFocusProgress.getTable();
			if(table!=null && table.size()>=1){
				for(WorkFocusProgressInfoVO workFocusProgressInfoVO:table){
					List<WorkFocusProgressEntity> data = workFocusProgressInfoVO.getData();
					workFocusProgressService.saveOrUpdateBatch(data);
					/*String problemInfo = workFocusProgressInfoVO.getProblemInfo();
					String workSug = workFocusProgressInfoVO.getWorkSug();*/
					String unitInfo = workFocusProgressInfoVO.getUnitInfo();
					String linkInfo = workFocusProgressInfoVO.getLinkInfo();
					if(Func.isNotEmpty(unitInfo)) {
						jdbcTemplate.update("update d_work_focus_progress set progress='"+workFocusProgressInfoVO.getUnitInfo()+"' where year='"+workFocusProgressInfoVO.getYear()+"' and month=13");
					}
					if(Func.isNotEmpty(linkInfo)) {
						jdbcTemplate.update("update d_work_focus_progress set progress='"+workFocusProgressInfoVO.getLinkInfo()+"' where year='"+workFocusProgressInfoVO.getYear()+"' and month=14");
					}
				}
			}
		}else {
			R.fail("参数为null,无法保存");
		}
		return R.status(true);
	}
}
